/***************************************************************************

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

***************************************************************************
Copyright (C) 2005 by 
Pan Wojtas (Wojtek Sulewski)
wojteksulewski <at> op.pl
gg: 2087202

***************************************************************************/

#include "modules/notify/notify.h"
#include "modules/sound/sound.h"
#include "debug.h" 
#include "config_dialog.h"
#include "userbox.h"
#include "message_box.h"
#include "config_file.h"

#include "iwait4u.h"
#include "iwait4uGUI.cpp"

Waiting *iwait4u_module;

extern "C" int iwait4u_init()
{
	kdebugf();
	
	iwait4u_module = new Waiting();
	
	kdebugf2();
	return 0;
}

extern "C" void iwait4u_close()
{
	kdebugf();
	
	delete iwait4u_module;

	kdebugf2();
}



Waiting::Waiting()
{
// wczytuje nasz uin
	OurUin = config_file.readNumEntry("General", "UIN");
	
// wczytuje, czy ma uywa dwikw
	UseSounds = config_file.readBoolEntry("Iwait4u", "iwait4u_use_sounds", false);
	RepeatSounds = config_file.readBoolEntry("Iwait4u", "iwait4u_repeat_sounds");
	SoundPath = config_file.readEntry("Iwait4u", "iwait4u_sound_path");
	TimeBetweenSounds = config_file.readUnsignedNumEntry("Hints","iwait4u_sounds_delay");
	if (TimeBetweenSounds == 0)
		TimeBetweenSounds = 3000;

// aduje z pliku list osb na ktre czekalimi w ostatniej sesji
	loadTrackedList();
	loadAlwaysTrackedList();

// tworzy GUI
	createGUI();
	
// teraz - po stworzeniu GUI przypisuje PosInMenu jak inn warto, ktrej uywa removeItem i setItemChecked.
	PosInMenu = UserBox::userboxmenu->getItem(tr("Wait for this person"));

// sprawdza, czy kto, na kogo czekamy nie jest dostpny w momencie uruchamiania moduu.
	if (!gadu->status().isOffline())
		checkOnConnection();

// czy wywietlenie popup z metod popupMenu
	connect(UserBox::userboxmenu, SIGNAL(popup()), this, SLOT(popupMenu()));
// userStatusChanged podwjnie, bo gadu nie informuje o niewidocznych, a notify o zaraz wracam -> dostpny
	connect(gadu, SIGNAL(userStatusChanged(const UserListElement&, const UserStatus&, bool)), this, SLOT(userStatusChanged(const UserListElement&, const UserStatus&)));
	connect(notify, SIGNAL(userStatusChanged(const UserListElement&, const UserStatus&)), this, SLOT(userStatusChanged_notify(const UserListElement&, const UserStatus&)));
}

Waiting::~Waiting()
{
// usuwa GUI
	destroyGUI();

// nie zapisuje list, bo one same si zapisuj w momencie zmiany ;)

	disconnect(UserBox::userboxmenu, SIGNAL(popup()), this, SLOT(popupMenu()));
	disconnect(gadu, SIGNAL(userStatusChanged(const UserListElement&, const UserStatus&, bool)), this, SLOT(userStatusChanged(const UserListElement&, const UserStatus&)));
	disconnect(notify, SIGNAL(userStatusChanged(const UserListElement&, const UserStatus&)), this, SLOT(userStatusChanged_notify(const UserListElement&, const UserStatus&)));
}

void Waiting::checkOnConnection(void)
{
	kdebugf();
	CONST_FOREACH(i, userlist)
	{
		if (!(*i).status().isOffline())
		{
			if (isOnTrackedList((*i).uin()))
				showMessage((*i), tr(" - last time, you have been waiting for this user. Now he is online!"), false);

			else if (isAlwaysTracked((*i).uin()))
				showMessage((*i), tr(" is online!!!"), true);
		}
	}

	kdebugf2();
}

void Waiting::userStatusChanged(const UserListElement &user, const UserStatus &oldStatus)
{
	kdebugf();
// 	if (!user.status().isInvisible())
// 	{
		if (isOnTrackedList(user.uin()))
		{
			if (user.status().isOnline() || user.status().isBusy())
				showMessage(user, tr(" is online!!!"), false);
	
			else if (!oldStatus.hasDescription() && user.status().isOffline() && user.status().hasDescription())
				showMessage(user, tr(" has changed his status 'offline' to 'offline with description' (can be invisible)!"), false);
	
			else if (oldStatus.hasDescription() && user.status().isOffline() && !user.status().hasDescription())
				showMessage(user, tr(" has changed his status 'offline with description' to 'offline' (can be invisible)!"), false);
	
			else if (oldStatus.hasDescription() && user.status().isOffline() && user.status().hasDescription())
				showMessage(user, tr(" has changed his description (can be invisible)!"), false);
	
	
		} else if (isAlwaysTracked(user.uin()))		// else zapobiega podwjnemu komunikatowi, jeli kto jest i na Tracked i na always
		{						// moe si tak zdarzy, jeli najpierw zosta dodany do Tracked, a pniej do always...
			if (user.status().isOnline() || user.status().isBusy())
				showMessage(user, tr(" is online!!!"), true);
	
			else if (!oldStatus.hasDescription() && user.status().isOffline() && user.status().hasDescription())
				showMessage(user, tr(" has changed his status 'offline' to 'offline with description' (can be invisible)!"), true);
	
			else if (oldStatus.hasDescription() && user.status().isOffline() && !user.status().hasDescription())
				showMessage(user, tr(" has changed his status 'offline with description' to 'offline' (can be invisible)!"), true);
	
			else if (oldStatus.hasDescription() && user.status().isOffline() && user.status().hasDescription())
				showMessage(user, tr(" has changed his description (can be invisible)!"), true);
		}
//	}

	kdebugf2();
}

void Waiting::userStatusChanged_notify(const UserListElement &user, const UserStatus &oldStatus)
{
	kdebugf();
	if (user.status().isInvisible())
	{
		if (isOnTrackedList(user.uin()))
		{
			showMessage(user, tr(" is invisible!!!"), false);
		}
		else if (isAlwaysTracked(user.uin()))
		{
			showMessage(user, tr(" is invisible!!!"), true);
		}
	}

	kdebugf2();
}

void Waiting::wait4Person(void)
{
	kdebugf();

	UserList users;
	UserBox *activeUserBox = UserBox::getActiveUserBox();
	if (activeUserBox == NULL)
	{
		return;
	}
	users = activeUserBox->getSelectedUsers();
	
	bool remove_all_from_tracked = false;

	for (UserList::ConstIterator i = users.begin(); i != users.end(); ++i)
	{
		if (isOnTrackedList((*i).uin()))
		{
			remove_all_from_tracked = true;
			break;			// wystarczy, e znajdzie jednego
		}
	}

	if (remove_all_from_tracked)
	{
		for (UserList::ConstIterator i = users.begin(); i != users.end(); ++i)
			removeFromTrackedList((*i));
	} else
	{
		for (UserList::ConstIterator i = users.begin(); i != users.end(); ++i)
			addToTrackedList((*i));
	}

	kdebugf2();
}

void Waiting::showMessage(const UserListElement &user, QString message, bool always)
{
	kdebugf();

	MessageBox *msg = new MessageBox(QString("<B>%1</B>").arg(QStyleSheet::escape(user.altNick())) + message, MessageBox::OK, false);
	QString icon = user.status().name().remove(" (d.)");		// usuwa description z nazwy statusu
	msg->setIcon(icons_manager.loadIcon(icon));			// aduje ikonk pokazujc, jaki status ma teraz user. name zwraca QString'a
	msg->show();							// nie ma delete, bo okienko samo si zniszczy w momencie nacinicia <OK>

	if (UseSounds)
	{
		if (RepeatSounds)
		{
			connect(msg, SIGNAL(okPressed()), this, SLOT(stopPlaingSound()));
			AgainSound = true;
			repeatSound();
		}
		else
			sound_manager->play(SoundPath, true);
	}

	if (!always)		// w sumie nic by si nie stao bez tego, ale...
		removeFromTrackedList(user);

	kdebugf2();
}




/**
	tracked
**/


void Waiting::loadTrackedList()
{
	QString loaded_str = config_file.readEntry("Iwait4u", "iwait4u_TrackedList");
	QStringList loaded_list = QStringList::split(",", loaded_str);
	
	int uin_int;
	bool ok;
	
	for (QStringList::ConstIterator i = loaded_list.begin(); i != loaded_list.end(); ++i)
	{
		uin_int = (*i).toInt(&ok, 10);
		if (ok)
			Tracked.append(uin_int);
		else
			kdebugm(KDEBUG_PANIC, "nie udao si zamieni QString na int");
	}
}

void Waiting::saveTrackedList()
{
	QStringList list_to_save;
	
	for (UinsList::ConstIterator i = Tracked.begin(); i != Tracked.end(); ++i)
	{
		QString str_uin;
		str_uin.sprintf("%d", (*i));
		list_to_save.append(str_uin);
	}
	config_file.writeEntry("Iwait4u", "iwait4u_TrackedList", list_to_save.join(","));
	config_file.sync();
}

void Waiting::addToTrackedList(const UserListElement &user)
{
	Tracked.append(user.uin());
	saveTrackedList();
}

void Waiting::removeFromTrackedList(const UserListElement &user)
{
	Tracked.remove(user.uin());
	saveTrackedList();
}

bool Waiting::isOnTrackedList(UinType uin)
{
	return Tracked.findIndex(uin)!=-1;		// findIndex zwraca -1, jeli nie znalaz...
}




/**
	ALWAYS tracked
**/


void Waiting::loadAlwaysTrackedList()
{
	QString loaded_str = config_file.readEntry("Iwait4u", "iwait4u_AlwaysTrackedList");
	QStringList loaded_list = QStringList::split(",", loaded_str);
	
	int uin_int;
	bool ok;
	
	for (QStringList::ConstIterator i = loaded_list.begin(); i != loaded_list.end(); ++i)
	{
		uin_int = (*i).toInt(&ok, 10);
		if (ok)
			AlwaysTracked.append(uin_int);
		else
			kdebugm(KDEBUG_PANIC, "nie udao si zamieni QString na int");
	}
}

void Waiting::saveAlwaysTrackedList()
{
	QStringList list_to_save;
	
	for (UinsList::ConstIterator i = AlwaysTracked.begin(); i != AlwaysTracked.end(); ++i)
	{
		QString str_uin;
		str_uin.sprintf("%d", (*i));
		list_to_save.append(str_uin);
	}
	config_file.writeEntry("Iwait4u", "iwait4u_AlwaysTrackedList", list_to_save.join(","));
	config_file.sync();
}

bool Waiting::isAlwaysTracked(UinType uin)
{
	return AlwaysTracked.findIndex(uin)!=-1;		// findIndex zwraca -1, jeli nie znalaz...
}




/**
	sounds
**/


void Waiting::stopPlaingSound(void)
{
	AgainSound = false;
}

void Waiting::repeatSound(void)
{
	sound_manager->play(SoundPath, true);
	if (AgainSound)
		QTimer::singleShot(TimeBetweenSounds, this, SLOT(repeatSound()));
}
